import requests
import json

def call_opener(text, opener_method): 
    template_url = 'http://opener.olery.com/{}'
    return requests.post(template_url.format(opener_method), data = {'input' : text}).text

def chained_call(text, opener_methods): 
    for opener_method in opener_methods:
        text = call_opener(text, opener_method)
    return text    
            
def langid(text):
    return call_opener(text, 'language-identifier')

def pos_tag(text):
    return call_opener(call_opener(langid(text), 'tokenizer'), 'pos-tagger')

def ned(text):
    return call_opener(call_opener(pos_tag(text), 'ner'), 'ned')

def kaf2json(text):
    return call_opener(text, 'kaf2json')

def json_ned(text):
    return json.loads(kaf2json(ned(text)))

def get_entities(text):
    return sorted(json_ned(text)['entities'].values(), key = lambda x: int(x['terms'][0][1:]))

def pretty_print_entities(text):
    print(json.dumps(get_entities(text), indent=4, separators=(',', ': ')))

# def reduced_get_entities(text):
#     entities = get_entities(text)
#     texts = set([e['text'] for e in entities])
#     should_disappear = set([x for x in texts if any(x in y for y in texts if x != y)])
#     return [e for e in entities if e['text'] not in should_disappear and e['type'] in ['PERSON', 'ORGANIZATION']]
    
    
    
    
####### clues:

# 'emphasize', 'tell', 'said', 'co-founder', 'CEO', 'managing director of', 'watchmaker' <name>,

# <quote> <person> <role> <company> <said/told>

# 'partnering'
# <C> engineers, <C> employees

# 'CapCap' invests

# <the> <thing>

# company, a <location> based

# find all indicators
# all single capital words not starting a sentence --> companies
# resolve 






# jruby -S language-identifier
# jruby -S tokenizer
# jruby -S pos-tagger
# jruby -S tree-tagger
# jruby -S polarity-tagger
# jruby -S property-tagger
# jruby -S constituent-parser
# jruby -S ner
# jruby -S coreference
# jruby -S ned
# jruby -S opinion-detector
# jruby -S opinion-detector-basic
# jruby -S kaf2json
# jruby -S outlet
# jruby -S scorer


# echo "They're offering $1.5 million in grants, and their first project is a Science Learning Challenge. That's for companies and nonprofits that are building tech to help students develop science and engineering skills. They'll pick up to 15 challenge winners with grants that range from $50,000 to $150,000. While we've seen literacy efforts and math kick up in the last few years, we'd like to see that same vibrancy in science education, said Stacey Childress, CEO of NewSchools Venture Fund. The ed tech that we like to fund is synergistic. It blends the best of technology with live instruction and immersive and engaging content. Along with the capital, NewSchools is working with a nonprofit research group called WestEd to provide design feedback and recommendations based on what educational research suggests will work best. They'll build small-scale studies to test usability and feasibility in the classroom. Childress said that NewSchools is in the process of raising another $60 million philanthropic fund over the next three years. They've had several funds over the organization's life since 1998 that range from a few million dollars to $75 million. Through that capital, the organization has supported about 442 charter schools throughout the country, serving 171,000 kids. Generally, they've backed educational entrepreneurs creating new schools, but they also want to fund technical teams building software and tools, as well. That's what this new accelerator is about. This is just the first part of what we're doing in this area, Childress said. We want to invest in ed tech and identify market gaps like the ones in science we were just talking about." | /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S language-identifier | /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S tokenizer | /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S pos-tagger | /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S ner




# txt="""They're offering $1.5 million in grants, and their first project is a Science Learning Challenge. That's for companies and nonprofits that are building tech to help students develop science and engineering skills. They'll pick up to 15 challenge winners with grants that range from $50,000 to $150,000. While we've seen literacy efforts and math kick up in the last few years, we'd like to see that same vibrancy in science education, said Stacey Childress, CEO of NewSchools Venture Fund. The ed tech that we like to fund is synergistic. It blends the best of technology with live instruction and immersive and engaging content. Along with the capital, NewSchools is working with a nonprofit research group called WestEd to provide design feedback and recommendations based on what educational research suggests will work best. They'll build small-scale studies to test usability and feasibility in the classroom. Childress said that NewSchools is in the process of raising another $60 million philanthropic fund over the next three years. They've had several funds over the organization's life since 1998 that range from a few million dollars to $75 million. Through that capital, the organization has supported about 442 charter schools throughout the country, serving 171,000 kids. Generally, they've backed educational entrepreneurs creating new schools, but they also want to fund technical teams building software and tools, as well. That's what this new accelerator is about. This is just the first part of what we're doing in this area, Childress said. We want to invest in ed tech and identify market gaps like the ones in science we were just talking about."""


# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-tree-tagger && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-polarity-tagger && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-property-tagger && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-constituent-parser && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-ner && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-coreference && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-ned && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-opinion-detector && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-opinion-detector-basic && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-kaf2json && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-outlet && \
# /Users/pascal/Downloads/jruby-9.0.0.0/bin/jruby -S gem install opener-scorer


# docker run -d -p 3306:3306 --name opener-mysql -e MYSQL_ROOT_PASSWORD=SECRET -e MYSQL_DATABASE=opener_development mysql
# docker run -e DB_PASS=SECRET -e DB_USER=root -e DB_HOST=192.168.59.103 -p 9292:9292 -ti bencomp/opener-basic




# split into separate files for DBs, or should I?

# Take care of backend for handle results
# for new project
# Create the databases
# Create the "default" document in plugins
# Create the design document

import json
import os
from sky.scraper import Scrape
from sky.crawler import crawl
from sky.crawler.crawling import NewsCrawler
from sky.helper import slugify

# ZODB specific
try:
    import transaction
    from BTrees.OOBTree import OOBTree
except NameError:
    print('Optional ZODB not possible as backend. Use `pip3 install ZODB zodbpickle`')


class CrawlPlugin():

    def __init__(self, project_name, server=None, plugin_name=None):
        self.project_name = project_name
        self.plugin_name = plugin_name
        self.crawl_config = None
        self.scrape_config = None
        self.data = {}
        self.documents = []
        self.template_dict = None
        self.server = server

    def get_default_plugin(self):
        pass

    def get_specific_plugin(self):
        pass

    def get_scrape_config(self):
        scrape_config = self.crawl_config.copy()

        scrape_config.update({
            'template_proportion': 0.09,
            'max_templates': 1000
        })

        return scrape_config

    def start_crawl(self):
        crawl.start(self.crawl_config)

    def scrape_data(self):
        # Create boilerplate recognizer
        skindex = Scrape(self.scrape_config)

        # Process all by removing boilerplate and extracting information
        return skindex.process_all(exclude_data=['cleaned', 'author'])

    def get_documents(self, maximum_number_of_documents=10000):
        pass

    def handle_results(self, to=None):
        pass

    def run(self, use_cache=False):
        # get default plugin
        self.crawl_config = self.get_default_plugin()

        # apply this specific plugin
        self.crawl_config.update(self.get_specific_plugin())

        self.scrape_config = self.get_scrape_config()
        if not use_cache:
            self.start_crawl()
        self.data = self.scrape_data()
        self.handle_results()

    def get_bad_summary(self, force_get_documents=False, n=5):
        if not self.documents or force_get_documents:
            self.documents = self.get_documents()
        title_sort = sorted(self.documents, key=lambda doc: len(doc['title']))
        body_sort = sorted(self.documents, key=lambda doc: len(' '.join(doc['body'])))
        date_sort = sorted(self.documents, key=lambda doc: len(doc['publish_date']))
        url_sort = sorted(self.documents, key=lambda doc: len(doc['url']))
        return {k: [(d['url'], d[k]) for d in sorted_type][:n]
                for k, sorted_type in zip(['title', 'body', 'publish_date', 'url'],
                                          [title_sort, body_sort, date_sort, url_sort])}


class CrawlFilePlugin(CrawlPlugin):

    def get_default_plugin(self):
        with open(os.path.join(self.server['plugins'], 'default')) as f:
            return json.load(f)

    def get_specific_plugin(self):
        with open(os.path.join(self.server['plugins'], self.plugin_name)) as f:
            specific_config = json.load(f)
        return specific_config

    def handle_results(self, to="file"):
        for res in self.data:
            with open(os.path.join(self.server['documents'], slugify(res['url'])), 'w') as f:
                json.dump(res, f)

    def get_documents(self, maximum_number_of_documents=1000000):
        slugged_url = slugify(self.plugin_name)
        results = []
        for num, fn in enumerate(os.listdir(self.server['documents'])):
            if num == maximum_number_of_documents:
                break
            if slugged_url in fn:
                with open(os.path.join(self.server['documents'], fn)) as f:
                    results.append(json.load(f))
        return results

    def get_seen_urls(self):
        slugged_url = slugify(self.plugin_name)
        seen_urls = set()
        for fn in os.listdir(self.server['documents']):
            if slugged_url in fn:
                with open(os.path.join(self.server['documents'], fn)) as f:
                    seen_urls.add(json.load(f)['url'])
        return seen_urls

    def save_config(self, config):
        with open(os.path.join(self.server['plugins'], self.plugin_name), 'w') as f:
            json.dump(config, f)


class CrawlCloudantPlugin(CrawlPlugin):

    def __init__(self, project_name, server, plugin_name):
        super(CrawlCloudantPlugin, self).__init__(project_name, server, plugin_name)
        self.dbs = {x: self.server.database(self.project_name + '-crawler-' + x) for x in
                    ['plugins', 'documents', 'template_dict']}

    def get_default_plugin(self):
        return self.dbs['plugins'].get('default').json()

    def get_specific_plugin(self):
        return self.dbs['plugins'].get(self.plugin_name).json()

    def handle_results(self, to="cloudant"):
        if to == "cloudant":
            for url_id in self.data:
                self.data[url_id]['_id'] = slugify(url_id)
            self.dbs['documents'].bulk_docs(*list(self.data.values()))

    def get_documents(self, maximum_number_of_documents=1000000):
        # now just to add the host thing ??????????????
        query = 'query={}'.format(self.plugin_name)
        params = '?include_docs=true&limit={}&{}'.format(maximum_number_of_documents, query)
        return [x['doc'] for x in self.dbs['documents'].all_docs().get(params).json()['rows']
                if self.plugin_name in x['doc']['url']]

    def get_seen_urls(self):
        params = '?query={}'.format(self.plugin_name)
        udocs = self.dbs['documents'].design('urlview').view('view1').get(params).json()['rows']
        return set([udoc['key'] for udoc in udocs])

    def save_config(self, config):
        doc = self.dbs['plugins'].get(self.plugin_name).json()
        if 'error' in doc:
            doc = {}
        doc.update(config)
        self.dbs['plugins'][self.plugin_name] = doc


class CrawlElasticSearchPlugin(CrawlPlugin):

    def __init__(self, project_name, server, plugin_name):
        super(CrawlElasticSearchPlugin, self).__init__(project_name, server, plugin_name)
        self.es = server

    def get_default_plugin(self):
        return self.es.get(id='default', doc_type='plugin',
                           index=self.project_name + "-crawler-plugins")['_source']

    def get_specific_plugin(self):
        return self.es.get(id=self.plugin_name, doc_type='plugin',
                           index=self.project_name + "-crawler-plugins")['_source']

    def handle_results(self, to="cloudant"):
        for url_id in self.data:
            doc_id = slugify(url_id)
            self.es.index(id=doc_id, body=self.data[url_id], doc_type='document',
                          index=self.project_name + "-crawler-documents")
        print(to)

    def get_documents(self, maximum_number_of_documents=1000000):
        query = {"query": {"wildcard": {"url": "*{}*".format(self.plugin_name)}}}
        res = self.es.search(body=query, doc_type='document',
                             index=self.project_name + "-crawler-documents")
        return res['hits']['hits']

    def get_seen_urls(self):
        query = {"query": {"wildcard": {"url": "*{}*".format(self.plugin_name)}}, "fields": "url"}
        res = self.es.search(body=query, doc_type='document',
                             index=self.project_name + "-crawler-documents")
        return set([x['fields']['url'][0] for x in res['hits']['hits']])

    def save_config(self, config):
        self.es.index(id=self.plugin_name, body=config, doc_type='plugin',
                      index=self.project_name + "-crawler-plugins")


class CrawlZODBPlugin(CrawlPlugin):

    def get_default_plugin(self):
        return self.server['plugins']['default']

    def get_specific_plugin(self):
        return self.server['plugins'][self.plugin_name]

    def handle_results(self, to="cloudant"):
        for url_id in self.data:
            self.server['documents'][slugify(url_id)] = self.data[url_id]
        transaction.commit()
        print(to)

    def get_documents(self, maximum_number_of_documents=1000000):
        return self.server['documents']

    def get_seen_urls(self):
        return set([self.server['documents'][s]['url'] for s in self.server['documents']
                    if self.plugin_name in self.server['documents'][s]['url']])

    def save_config(self, config):
        self.server['plugins'][self.plugin_name] = config
        transaction.commit()


class CrawlPluginNews(CrawlPlugin):
    import ast

    def save_data_while_crawling(self, data):
        raise NotImplementedError('save_data_while_crawling required')

    def get_template_dict(self):
        raise NotImplementedError('get_template_dict required')

    def save_template_dict(self, templated_dict):
        raise NotImplementedError('save_template_dict required')

    def get_seen_urls(self):
        raise NotImplementedError('seen_urls required')

    def run(self, use_cache=False):
        # get default plugin
        self.crawl_config = self.get_default_plugin()

        # apply this specific plugin
        self.crawl_config.update(self.get_specific_plugin())

        # add the already visited urls to the config
        seen_urls = self.get_seen_urls()
        self.crawl_config['seen_urls'] = seen_urls

        # inherits from crawl_config
        self.scrape_config = self.get_scrape_config()

        # add the template for this crawl_plugin to the scraping config
        self.scrape_config['template_dict'] = self.get_template_dict()

        # separate out the save data while crawling and the newscraler
        templated_dict = crawl.start(self.scrape_config, NewsCrawler,
                                     self.save_data_while_crawling)
        self.save_template_dict(templated_dict)


class CrawlFilePluginNews(CrawlFilePlugin, CrawlPluginNews):

    def save_data_while_crawling(self, data):
        with open(os.path.join(self.server['documents'], slugify(data['url'])), 'w') as f:
            json.dump(data, f)

    def get_template_dict(self):
        temp_dict_fn = os.path.join(self.server['template_dict'], self.plugin_name)
        if os.path.isfile(temp_dict_fn):
            with open(temp_dict_fn) as f:
                template_dict = {self.ast.literal_eval(k): v for k, v in json.load(f).items()
                                 if not k.startswith('_')}
        else:
            template_dict = {}

        return template_dict

    def save_template_dict(self, templated_dict):
        if templated_dict:
            temp_dict_fn = os.path.join(self.server['template_dict'], self.plugin_name)
            with open(temp_dict_fn, 'w') as f:
                json.dump({repr(k): v for k, v in templated_dict.items()}, f)


class CrawlZODBPluginNews(CrawlZODBPlugin, CrawlPluginNews):

    def save_data_while_crawling(self, data):
        self.server['documents'][slugify(data['url'])] = data
        transaction.commit()

    def get_template_dict(self):
        if ('template-dict' not in self.server or
                self.plugin_name not in self.server['template-dict']):
            template_dict = OOBTree()
        else:
            template_dict = self.server['template-dict'][self.plugin_name]
        return template_dict

    def save_template_dict(self, templated_dict):
        if templated_dict:
            self.server['template-dict'][self.plugin_name] = OOBTree(templated_dict)
            transaction.commit()


class CrawlElasticSearchPluginNews(CrawlElasticSearchPlugin, CrawlPluginNews):

    def save_data_while_crawling(self, data):
        self.es.index(index=self.project_name + "-crawler-documents", doc_type='document',
                      id=slugify(data['url']), body=data)

    def get_template_dict(self):
        try:
            templ_dict = self.es.get(index=self.project_name + "-crawler-template-dict",
                                     doc_type='template-dict',
                                     id=self.plugin_name)['_source']
            templ_dict = {self.ast.literal_eval(k): v for k, v in templ_dict.items()}
        # In acually catching the NotFoundError (for which I do not want to depend
        # on importing the elasticsearch package)
        # pylint: disable=bare-except
        except:
            templ_dict = {}
        return templ_dict

    def save_template_dict(self, templ_dict):
        if templ_dict:
            try:
                self.es.index(index=self.project_name + "-crawler-template-dict",
                              doc_type='template-dict', id=self.plugin_name,
                              body=json.dumps({repr(k): v for k, v in templ_dict.items()}))
            # In acually catching the NotFoundError (for which I do not want to depend
            # on importing the elasticsearch package)
            # pylint: disable=bare-except
            except:
                self.es.update(
                    index=self.project_name + "-crawler-template-dict",
                    doc_type='template-dict', id=self.plugin_name,
                    body=json.dumps({"doc": {repr(k): v for k, v in templ_dict.items()}}))


class CrawlCloudantPluginNews(CrawlCloudantPlugin, CrawlPluginNews):

    def save_data_while_crawling(self, data):
        self.dbs['documents'][slugify(data['url'])] = data

    def get_template_dict(self):
        template_dict = self.dbs['template-dict'].get(self.plugin_name).json()
        if 'error' in template_dict:
            template_dict = {}
        else:
            template_dict = {self.ast.literal_eval(k): v for k, v in template_dict.items()
                             if not k.startswith('_')}
        return template_dict

    def save_template_dict(self, templated_dict):
        if templated_dict:
            doc = self.dbs['template-dict'].get(self.plugin_name).json()
            if 'error' in doc:
                doc = {}
            doc.update({repr(k): v for k, v in templated_dict.items()})
            self.dbs['template-dict'][self.plugin_name] = doc



## <figure> should also be considered
## background-url should also be allowed

## Also return most likely logo (logo, favicon??, first image?, icon)

def get_images(tree, wrong_imgs = None): 
    if wrong_imgs is None:
        wrong_imgs = ['adsense', 'icon', 'logo', 'advert', 'toolbar', 'footer', 'layout', 'banner'] 
    img_candidates = tree.xpath('//img[string-length(@src) > 3]')
    img_candidates += tree.xpath('//meta[contains(@property, "image")]')
    leftover = []
    for img_candidate in img_candidates: 
        if any([any([w in img_candidate.attrib[a] for w in wrong_imgs]) for a in img_candidate.attrib]):
            continue
        if img_candidate.tag == 'img':
            try:
                if 'height' in img_candidate.attrib and int(img_candidate.attrib['height']) < 25:
                    continue 
                if 'width' in img_candidate.attrib and int(img_candidate.attrib['width']) < 25:
                    continue 
            except ValueError: 
                pass
        leftover.append(img_candidate) 
    return leftover



DEFAULT_CRAWL_CONFIG = {
    # Required
    'seed_urls': [

    ],

    'collections_path': '',

    'collection_name': '',

    # Optional

    'crawl_filter_regexps': [

    ],

    'crawl_required_regexps': [
    ],

    'index_filter_regexps': [

    ],

    'index_required_regexps': [

    ],

    'max_redirects_per_url': 10,

    'max_tries_per_url': 1,

    'max_workers': 5,

    'max_saved_responses': 20,

    'login_url': '',
    'login_data': {},

    # Unimplemented
    'max_hops': 10
}

PRODUCTION_CRAWL_CONFIG = DEFAULT_CRAWL_CONFIG.copy()

PRODUCTION_CRAWL_CONFIG.update({
    'max_saved_responses': 1000000000,
    'max_hops': 100,
    'max_workers': 100
})



########## 1. Setup ##################################################
PROJECT_NAME = 'testproj'

######### 2a. File      ##############################################

from sky.crawler_services import CrawlFileService
from sky.crawler_plugins import CrawlFilePluginNews

storage_object = {'path': '/Users/pascal/sky_collections/'}

cs = CrawlFileService(PROJECT_NAME, storage_object, CrawlFilePluginNews)

######### 2b. Cloudant  ##############################################

import cloudant
from sky.crawler_services import CrawlCloudantService
from sky.crawler_plugins import CrawlCloudantPluginNews

with open('cloudant.username') as f:
    USERNAME = f.read()
with open('cloudant.password') as f:
    PASSWORD = f.read()
account = cloudant.Account(USERNAME)
account.login(USERNAME, PASSWORD)

cs = CrawlCloudantService(PROJECT_NAME, account, CrawlCloudantPluginNews)

######### 2c. ElasticSearch ##########################################

import elasticsearch

from sky.crawler_services import CrawlElasticSearchService
from sky.crawler_plugins import CrawlElasticSearchPluginNews

es = elasticsearch.Elasticsearch([{'host': 'localhost', 'port': 9200}])

cs = CrawlElasticSearchService(PROJECT_NAME, es, CrawlElasticSearchPluginNews)

######### 2d. ZODB ###################################################

import ZODB.FileStorage
from sky.crawler_services import CrawlZODBService
from sky.crawler_plugins import CrawlZODBPluginNews

fname = '/Users/pascal/sky_collections/zodb/{}.fs'.format(PROJECT_NAME)
storage = ZODB.FileStorage.FileStorage(fname)

cs = CrawlZODBService(PROJECT_NAME, storage, CrawlZODBPluginNews)

######### 3. Add config files to the database ########################
from sky.configs import DEFAULT_CRAWL_CONFIG

default = cs.get_crawl_plugin('default')
default.save_config(DEFAULT_CRAWL_CONFIG)

bbc_config = {
    'seed_urls': ['http://www.bbc.com/news/world/europe'],
    'crawl_required_regexps': ['europe'],
    'index_required_regexps': ['news/world-europe-'],
}

bbc = cs.get_crawl_plugin('bbc.com')
bbc.save_config(bbc_config)

######## 4. Start crawling ###########################################
cs.run('bbc.com')



import re
import json
import dateutil.parser
import datetime
import os

from sky.helper import get_text_and_tail

from pkg_resources import resource_filename
fname = os.path.abspath(resource_filename('sky.data', 'date_translation_table.json'))

with open(fname) as f:
    date_translation_table = json.load(f)
    uppered = {x.title(): date_translation_table[x] for x in date_translation_table}
    date_translation_table.update(uppered)


class NoDefaultDate(object):
    """ Credits http://stackoverflow.com/a/18242643/1575066 """

    def replace(self, **fields):
        if any(f not in fields for f in ('year', 'month', 'day')):
            return None
        return datetime.datetime(2000, 1, 1).replace(**fields)


def patched_dateutil_parse(v, fuzzy):
    _actual = dateutil.parser.parse(v, default=NoDefaultDate(), fuzzy=fuzzy)
    if _actual is not None:
        # pylint: disable=E1101
        return _actual.date()
    else:
        return False


def date_translation(txt, lang):
    if lang in date_translation_table:
        for month in date_translation_table[lang]:
            txt = txt.replace(month, date_translation_table[lang][month])
    return txt


def get_text_date(v, fuzzy=False):
    try:
        d = patched_dateutil_parse(v, fuzzy)
        return d
    except (ValueError, OverflowError, TypeError, AttributeError):
        return False


def within_years(d):
    return re.search(r'\b(19[89][0-9]|20[0-4][0-9])\b', d)


def get_dates(tree, lang='en'):
    # make this faster, its friggin slow (stupid fuzzy matching)
    hard_dates = []
    soft_dates = []
    fuzzy_hard_dates = []
    fuzzy_soft_dates = []
    meta_nodes = tree.xpath('//head/meta')

    goods = ['ublish', 'ublicat', 'date', 'time']

    for option in goods:

        for meta in meta_nodes:
            if not any([option in a for a in meta.values()]):
                continue

            for attr in meta.attrib:
                soft_dates.append(get_text_date(date_translation(meta.attrib[attr], lang)))

    for num, node in enumerate(tree.iter()):
        candi_dates = [v for k, v in node.items() if v and any([x in k for x in goods])]
        for v in candi_dates:
            if within_years(v):
                if lang != 'en':
                    v = date_translation(v, lang)
                d = get_text_date(v)
                if d:
                    soft_dates.append(d)
                else:
                    fuzzy_soft_dates.append(get_text_date(v, fuzzy=True))

        # hard date
        tailtext = get_text_and_tail(node).strip()
        if tailtext and within_years(tailtext):
            if lang != 'en':
                tailtext = date_translation(tailtext, lang)
            hard_date = get_text_date(tailtext)
            if hard_date:
                hard_dates.append((num, hard_date))
            else:
                fuzzy_hard_dates.append((num, get_text_date(tailtext, fuzzy=True)))

    soft_dates = set(soft_dates)
    fuzzy_soft_dates = set(x for x in fuzzy_soft_dates if x)
    fuzzy_hard_dates = [x for x in fuzzy_hard_dates if x]

    # Note that num and hd get switched here
    hardest_dates = []
    not_hardest_dates = []
    for num, hd in hard_dates:
        if hd in soft_dates:
            hardest_dates.append((hd, num))
        else:
            not_hardest_dates.append((hd, num))

    fuzzy_hardest_dates = []
    for num, hd in fuzzy_hard_dates:
        if hd in fuzzy_soft_dates:
            fuzzy_hardest_dates.append((hd, num))
        else:
            not_hardest_dates.append((hd, num))

    # if nothing, then try simply fuzzy on each node, and otherwise non fuzzy
    non_fuzzy_any = []
    fuzzy_any = []
    if not any([hardest_dates, fuzzy_hardest_dates, not_hardest_dates, soft_dates]):
        # no leads, try to parse everything non fuzzy
        for num, node in enumerate(tree.iter()):
            non_fuzzy_text = get_text_date(node, fuzzy=False)
            if non_fuzzy_text:
                non_fuzzy_any.append((non_fuzzy_text, num))
            else:
                fuzzy_text = get_text_date(node, fuzzy=True)
                if fuzzy_text:
                    fuzzy_any.append((fuzzy_text, num))

    return hardest_dates, fuzzy_hardest_dates, not_hardest_dates, soft_dates, non_fuzzy_any, fuzzy_any



import argparse

from sky.view import view


def get_args_parser():
    p = argparse.ArgumentParser(description='sky is the limit.')
    subparsers = p.add_subparsers(dest="command")
    view_parser = subparsers.add_parser('view')
    view_parser.add_argument('-port', '-p', type=int, default=7900)
    view_parser.add_argument('-host', '-h', default="localhost")
    return p


def main():
    """ This is the function that is run from commandline with `gittyleaks` """
    args = get_args_parser().parse_args()
    if args.command == 'view':
        view.main(args.port)



from sky.helper import fscore

############## Multi

def generate_rule_dictionary():
    headers = ['h{}'.format(i) for i in range(1, 5)]
    #tags = ['strong', 'b', 'div']
    tags = []
    atts = ['id', 'class', '']
    res = {k : {kk : {} for kk in atts} for k in headers + tags} 
    it = 0
    for h in headers:
        for k in atts:
            it += 1
            res[h][k]['title'] = it 
            it += 1
            res[h][k][''] = it
            it += 1    
        res[h]['']['title'] = it
        it += 1    
        res[h][''][''] = it
    # for t in tags:
    #     for k in atts:
    #         it += 1
    #         res[t][k]['title'] = it 
    #         it += 1
    #         res[t][k][''] = 100 
    #         it += 1
    #     res[t]['']['title'] = it    
    #     it += 1
    #     if t != 'div':
    #         res[t][''][''] = it    

    return res

def get_score_from_title_dict(node, dc):
    tag_found = dc.get(node.tag, '')
    if tag_found:
        maxi = 100
        for attribute in node.attrib:
            key_found = dc[node.tag].get(attribute, '')
            if key_found:
                if 'title' in node.attrib[attribute]: 
                    maxi = min(maxi, dc[node.tag][attribute]['title'])
                else:    
                    maxi = min(maxi, dc[node.tag][attribute][''])
        #if maxi == 100:
            #maxi = dc[node.tag][''][''] 
        return maxi, node.text_content()
    else:    
        return 101, ''
            
def get_meta_titles(tree): 
    res = []
    head = tree.find('head')
    if head is not None:
        for xp in ['.//title/text()', './/meta[contains(@name, "title")]/@content', 
                   './/meta[contains(@property, "title")]/@content']:
            res.extend(head.xpath(xp))
    return res                

def sorted_title_candidates(tree, rule_dict):
    mins = []            
    for node in tree.iter(): 
        score, ele = get_score_from_title_dict(node, rule_dict) 
        stripped_ele = ele.strip()
        if stripped_ele and score != 101:
            mins.append((score, stripped_ele))

    return [x[1] for x in sorted(mins)]

rule_dc = generate_rule_dictionary()
    
def getRuleTitle(tree, rule_dict = rule_dc):
    texts = sorted_title_candidates(tree, rule_dict)
    metas = get_meta_titles(tree)
    maxs = 0
    ele = ''
    for x in texts:
        xs = x.lower().split()
        for y in metas:            
            ys = y.lower().split()
            newm = fscore(xs, ys)
            if newm > maxs:
                maxs = newm
                ele = x
    return ele

# tr = getQuickTree('http://www.bbc.com/news/world-africa-33049312')

# tr1 = getQuickTree('http://www.bbc.com/news/science-environment-33268180')

# tr2 = getQuickTree('http://www.nieuwsdumper.nl/nieuws/1454/eerste-volvo-fmx-410-8x4-tridem-betonmixer-voor-bck.html')

# tr3 = getQuickTree('http://www.nieuwsdumper.nl/nieuws/1671/eerste-doosan-vijf-serie-in-friesland-afgeleverd-aan-kor-de-boer.html')

# rule_dict = generate_rule_dictionary()

# get_title_from_text_meta(tr, rule_dict)

###################################################################

###########  EXCLUDE FUCKING DICT HERE TEMPLATE STYLE


###################################################################

# def getTitle(tree, returnBest = True):
#     # maybe lowercase wordsets, look at cases 37,38 and 53
#     # we are now in overfitting terrain
#     # consider regex on Title and title or just double
#     # lets not ignore dash in class contains
#     xpaths = ['//title', '//*[contains(@name, "title")]', '//h1[contains(@id, "title")]', '//h1[contains(@class, "title")]', '//h1[@*="title"]', '//*[contains(@id, "title")]',  '//meta[contains(@property, "title")]', '//meta[contains(@property, "title")]/@content', '//h1', '//h2', '//*[contains(@class, "title")]', '//*[@title]', '//h3', '//h4', '//strong', '//*[contains(@*, "headline")]', '//*[contains(@*, "Headline")]']
#     # xpaths = ['//title', '//meta[contains(@property, "title")]', '//*[contains(@name, "title")]', '//h1[contains(@id, "title")]', '//h1[contains(@class, "title")]', '//h1[@*="title"]', '//*[contains(@id, "title")]', '//*[@*="title"]', '//*[contains(@class, "title")]', '//*[@title]', '//h1', '//h2', '//h3', '//h4']
#     titles = []
#     for xp in xpaths:
#         xresults = tree.xpath(xp)
#         title = {}
#         for xresult in xresults:
#             if isinstance(xresult, lxml.html.HtmlElement):
#                 title['text'] = xresult.text_content().strip()
#             else:
#                 title['text'] = xresult   
#             if len(title['text']) < 3:
#                 continue
#             title['wordSet'] = set(x for x in title['text'].split() if len(x) > 1)
#             title['wordSetLength'] = len(title['wordSet'])
#             title['xpath'] = xp
#             titles.append(title)

#     if not returnBest:
#         return titles

#     t = titles[0]
#     for title in titles[1:]:
#         if len(t['wordSet'] & title['wordSet']) / len(t['wordSet']) > 0.3 and t['wordSetLength'] > title['wordSetLength']: 
#             break
#     else:
#         title = t
#     return title['text']
